@@ -14,7 +14,7 @@ inline double ceil_multi_unit(double x, double n) noexcept {
 }
 
 enum class Unit {YEAR, HALFYEAR, QUARTER, SEASON, BIMONTH, MONTH,
-                 WEEK, DAY, HOUR, MINUTE, SECOND, ASECOND};
+  WEEK, DAY, HOUR, MINUTE, SECOND, ASECOND};
 
 Unit name2unit(std::string unit_name) {
   if (unit_name == "asecond") return Unit::ASECOND;
@@ -36,28 +36,30 @@ Unit name2unit(std::string unit_name) {
 
 // used in time_floor exclusively
 template<typename T>
-inline double civil_time_to_posix(const T& ct,
-                                  const cctz::time_zone& tz,
-                                  const time_point& tp_orig,
-                                  const cctz::civil_second& cs_orig) noexcept {
+inline double ct2posix4floor(const T& ct,
+                             const cctz::time_zone& tz,
+                             const time_point& tp_orig,
+                             const cctz::civil_second& cs_orig) noexcept {
   cctz::time_zone::civil_lookup cl = tz.lookup(ct);
-  return civil_lookup_to_posix(cl, tz, tp_orig, cs_orig, Roll::BOUNDARY, 0.0);
+  return civil_lookup_to_posix(
+    cl, tz, tp_orig, cs_orig, DST(RollDST::BOUNDARY, RollDST::POST), 0.0);
 }
 
 // used for time_ceiling exclusively
 template<typename T>
-inline double civil_time_to_posix(const T& ct,
-                                  const cctz::time_zone& tz,
-                                  const time_point& tp_orig,
-                                  const cctz::civil_second& cs_orig,
-                                  const int N,
-                                  const bool check_boundary) noexcept {
+inline double ct2posix4ceiling(const T& ct,
+                               const cctz::time_zone& tz,
+                               const time_point& tp_orig,
+                               const cctz::civil_second& cs_orig,
+                               const int N,
+                               const bool check_boundary) noexcept {
   if (check_boundary && cs_orig == ct - N) {
     time_point tpnew = cctz::convert(cs_orig, tz);
     return tpnew.time_since_epoch().count();
   } else {
     cctz::time_zone::civil_lookup cl = tz.lookup(ct);
-    return civil_lookup_to_posix(cl, tz, tp_orig,cs_orig, Roll::BOUNDARY, 0.0);
+    return civil_lookup_to_posix(
+      cl, tz, tp_orig, cs_orig, DST(RollDST::BOUNDARY, RollDST::PRE), 0.0);
   }
 }
 
@@ -106,41 +108,41 @@ Rcpp::newDatetimeVector C_time_ceiling(const NumericVector dt,
      }
      case Unit::SECOND : {
        cctz::civil_second ct = cctz::civil_second(cctz::civil_minute(cs)) + CEIL_MULTI_UNIT(cs.second(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::MINUTE : {
        cctz::civil_minute ct = cctz::civil_minute(cctz::civil_hour(cs)) + CEIL_MULTI_UNIT(cs.minute(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::HOUR : {
        cctz::civil_hour ct = cctz::civil_hour(cctz::civil_day(cs)) + CEIL_MULTI_UNIT(cs.hour(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::DAY : {
        /* Rprintf("day:%d ceil:%d\n", cs.day(), CEIL_MULTI_UNIT1(cs.day(), N)); */
        cctz::civil_day ct = cctz::civil_day(cctz::civil_month(cs)) + CEIL_MULTI_UNIT1(cs.day(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::WEEK : {
        if (N != 1)
          Rf_warning("Multi-unit week ceiling is not supported; ignoring");
        cctz::civil_day ct = cctz::next_weekday(cctz::civil_day(cs), wday);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, 7, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, 7, check_boundary); break;
      }
      case Unit::SEASON : {
        cctz::civil_month ct = cctz::civil_month(cctz::civil_year(cs)) + CEIL_MULTI_UNIT(cs.month(), N) - 1;
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::HALFYEAR :
      case Unit::QUARTER :
      case Unit::BIMONTH :
      case Unit::MONTH : {
        cctz::civil_month ct = cctz::civil_month(cctz::civil_year(cs)) + CEIL_MULTI_UNIT1(cs.month(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
      case Unit::YEAR : {
        cctz::civil_year ct = cctz::civil_year(CEIL_MULTI_UNIT(cs.year(), N));
-       out[i] = civil_time_to_posix(ct, tz, tp, cs, N, check_boundary); break;
+       out[i] = ct2posix4ceiling(ct, tz, tp, cs, N, check_boundary); break;
      }
     }
   }
@@ -185,41 +187,41 @@ Rcpp::newDatetimeVector C_time_floor(const NumericVector dt,
      }
      case Unit::SECOND : {
        cctz::civil_second ct = cctz::civil_second(cctz::civil_minute(cs)) + FLOOR_MULTI_UNIT(cs.second(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::MINUTE : {
        cctz::civil_minute ct = cctz::civil_minute(cctz::civil_hour(cs)) + FLOOR_MULTI_UNIT(cs.minute(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::HOUR : {
        cctz::civil_hour ct = cctz::civil_hour(cctz::civil_day(cs)) + FLOOR_MULTI_UNIT(cs.hour(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::DAY : {
        cctz::civil_day ct = cctz::civil_day(cctz::civil_month(cs)) + FLOOR_MULTI_UNIT1(cs.day(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::WEEK : {
        if (N != 1)
          Rf_warning("Multi-unit week rounding is not supported; ignoring");
        // get the previous `wday` if `cs` is not already on `wday`
        cctz::civil_day ct = cctz::next_weekday(cctz::civil_day(cs), wday) - 7;
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::SEASON : {
        cctz::civil_month ct = cctz::civil_month(cctz::civil_year(cs)) + FLOOR_MULTI_UNIT(cs.month(), N) - 1;
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::HALFYEAR :
      case Unit::QUARTER :
      case Unit::BIMONTH :
      case Unit::MONTH : {
        cctz::civil_month ct = cctz::civil_month(cctz::civil_year(cs)) + FLOOR_MULTI_UNIT1(cs.month(), N);
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
      case Unit::YEAR : {
        cctz::civil_year ct = cctz::civil_year(FLOOR_MULTI_UNIT(cs.year(), N));
-       out[i] = civil_time_to_posix(ct, tz, tp, cs); break;
+       out[i] = ct2posix4floor(ct, tz, tp, cs); break;
      }
     }
   }
